From 1b991b5c66cc44b6f93a14c1c265683074224004 Mon Sep 17 00:00:00 2001
From: Cyrille Bagard <nocbos@gmail.com>
Date: Mon, 2 Oct 2023 09:14:50 +0200
Subject: Implement Base64 encoding for scan pattern modifications.

---
 configure.ac                                       |   4 +
 plugins/Makefile.am                                |   1 +
 plugins/encodings/Makefile.am                      |  64 +++
 plugins/encodings/base64.c                         | 137 ++++++
 plugins/encodings/base64.h                         |  43 ++
 plugins/encodings/core.c                           |  89 ++++
 plugins/encodings/core.h                           |  38 ++
 plugins/encodings/python/Makefile.am               |  20 +
 plugins/encodings/python/base64.c                  | 139 ++++++
 plugins/encodings/python/base64.h                  |  39 ++
 plugins/encodings/python/module.c                  |  87 ++++
 plugins/encodings/python/module.h                  |  38 ++
 plugins/encodings/python/rost/Makefile.am          |  14 +
 plugins/encodings/python/rost/base64.c             | 211 +++++++++
 plugins/encodings/python/rost/base64.h             |  45 ++
 plugins/encodings/python/rost/module.c             |  63 +++
 plugins/encodings/python/rost/module.h             |  38 ++
 plugins/encodings/rost/Makefile.am                 |  12 +
 plugins/encodings/rost/base64.c                    | 513 +++++++++++++++++++++
 plugins/encodings/rost/base64.h                    |  58 +++
 .../analysis/scan/patterns/modifiers/xor.c         |   6 +-
 src/analysis/scan/patterns/modifiers/hex.c         |   2 +-
 src/analysis/scan/patterns/modifiers/plain.c       |   2 +-
 src/analysis/scan/patterns/modifiers/rev.c         |   2 +-
 src/analysis/scan/patterns/modifiers/xor.c         |   4 +-
 src/common/szstr.h                                 |   5 +
 tests/analysis/scan/pyapi.py                       |  14 +
 27 files changed, 1680 insertions(+), 8 deletions(-)
 create mode 100644 plugins/encodings/Makefile.am
 create mode 100644 plugins/encodings/base64.c
 create mode 100644 plugins/encodings/base64.h
 create mode 100644 plugins/encodings/core.c
 create mode 100644 plugins/encodings/core.h
 create mode 100644 plugins/encodings/python/Makefile.am
 create mode 100644 plugins/encodings/python/base64.c
 create mode 100644 plugins/encodings/python/base64.h
 create mode 100644 plugins/encodings/python/module.c
 create mode 100644 plugins/encodings/python/module.h
 create mode 100644 plugins/encodings/python/rost/Makefile.am
 create mode 100644 plugins/encodings/python/rost/base64.c
 create mode 100644 plugins/encodings/python/rost/base64.h
 create mode 100644 plugins/encodings/python/rost/module.c
 create mode 100644 plugins/encodings/python/rost/module.h
 create mode 100644 plugins/encodings/rost/Makefile.am
 create mode 100644 plugins/encodings/rost/base64.c
 create mode 100644 plugins/encodings/rost/base64.h

diff --git a/configure.ac b/configure.ac
index 9de9abd..3cf68d3 100644
--- a/configure.ac
+++ b/configure.ac
@@ -659,6 +659,10 @@ AC_CONFIG_FILES([Makefile
                  plugins/dwarf/v2/Makefile
                  plugins/dwarf/v3/Makefile
                  plugins/dwarf/v4/Makefile
+                 plugins/encodings/Makefile
+                 plugins/encodings/python/Makefile
+                 plugins/encodings/python/rost/Makefile
+                 plugins/encodings/rost/Makefile
                  plugins/elf/Makefile
                  plugins/elf/python/Makefile
                  plugins/fmtp/Makefile
diff --git a/plugins/Makefile.am b/plugins/Makefile.am
index 7ce4229..3502cf0 100644
--- a/plugins/Makefile.am
+++ b/plugins/Makefile.am
@@ -20,6 +20,7 @@ SUBDIRS = \
 	dex \
 	dexbnf \
 	dwarf \
+	encodings \
 	elf \
 	fmtp \
 	itanium \
diff --git a/plugins/encodings/Makefile.am b/plugins/encodings/Makefile.am
new file mode 100644
index 0000000..6dd71fb
--- /dev/null
+++ b/plugins/encodings/Makefile.am
@@ -0,0 +1,64 @@
+
+lib_LTLIBRARIES = libencodings.la
+
+libdir = $(pluginslibdir)
+
+
+if BUILD_PYTHON_PACKAGE
+
+RUN_PATH = -Wl,-rpath,'$$ORIGIN/../chrysalide-libs:$$ORIGIN'
+
+else
+
+RUN_PATH = -Wl,-rpath,'$$ORIGIN'
+
+endif
+
+if BUILD_PYTHON3_BINDINGS
+
+PYTHON3_LIBADD = python/libencodingspython.la
+
+if BUILD_DISCARD_LOCAL
+
+if BUILD_PYTHON_PACKAGE
+PYTHON3_RUN_PATH = -Wl,-rpath,'$$ORIGIN/..'
+else
+PYTHON3_RUN_PATH = -Wl,-rpath,'$$ORIGIN'
+endif
+
+else
+
+PYTHON3_RUN_PATH = -Wl,-rpath,$(abs_top_srcdir)/plugins/pychrysalide/.libs
+
+endif
+
+PYTHON3_LDFLAGS = $(PYTHON3_RUN_PATH) -L$(top_srcdir)/plugins/pychrysalide/.libs -l:pychrysalide.so
+
+PYTHON3_SUBDIRS = python
+
+endif
+
+
+libencodings_la_SOURCES =					\
+	base64.h base64.c						\
+	core.h core.c
+
+libencodings_la_LIBADD =					\
+	$(PYTHON3_LIBADD)						\
+	rost/libencodingsrost.la
+
+libencodings_la_CFLAGS = $(TOOLKIT_CFLAGS) $(LIBXML_CFLAGS) -I$(top_srcdir)/src
+
+libencodings_la_LDFLAGS =					\
+	-avoid-version							\
+	-L$(top_srcdir)/src/.libs -lchrysacore	\
+	-L$(top_srcdir)/plugins/pe/.libs -lpe	\
+	$(RUN_PATH) $(PYTHON3_LDFLAGS)
+
+
+devdir = $(includedir)/chrysalide/$(subdir)
+
+dev_HEADERS = $(libencodings_la_SOURCES:%c=)
+
+
+SUBDIRS = $(PYTHON3_SUBDIRS) rost
diff --git a/plugins/encodings/base64.c b/plugins/encodings/base64.c
new file mode 100644
index 0000000..61b88e0
--- /dev/null
+++ b/plugins/encodings/base64.c
@@ -0,0 +1,137 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.c - calculs d'encodages en base 64
+ *
+ * Copyright (C) 2020 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+
+#include "base64.h"
+
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : input    = données d'entrée à traiter.                       *
+*                output   = données sortantes constituées. [OUT]              *
+*                alphabet = alphabet d'encodage mis à disposition.            *
+*                                                                             *
+*  Description : Procède à l'encodage d'un contenu en base 64.                *
+*                                                                             *
+*  Retour      : Bilan de l'opération : true en cas de réussite, false sinon. *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+bool _base64_encode(const sized_binary_t *input, sized_string_t *output, const sized_string_t *alphabet)
+{
+    bool result;                            /* Bilan à retourner           */
+    const bin_t *src;                       /* Tête de lecture #1          */
+    const bin_t *alpha;                     /* Tête de lecture #2          */
+    bin_t *iter;                            /* Tête d'écriture             */
+    size_t i;                               /* Boucle de parcours          */
+
+    result = (alphabet->len == 64);
+    if (!result) goto exit;
+
+    /* Création du réceptacle */
+
+    output->len = input->len * 4 / 3;
+
+    if (input->len % 3 != 0)
+        output->len++;
+
+    output->data = malloc((output->len + 1) * sizeof(bin_t));
+
+    /* Encodage du corps du message */
+
+    src = input->static_bin_data;
+    alpha = alphabet->static_bin_data;
+
+    iter = output->bin_data;
+
+    if (input->len > 2)
+        for (i = 0; i < (input->len - 2); i += 3)
+        {
+            *iter++ = alpha[(src[i] >> 2) & 0x3f];
+
+            *iter++ = alpha[((src[i] & 0x03) << 4) | ((src[i + 1] & 0xf0) >> 4)];
+
+            *iter++ = alpha[((src[i + 1] & 0x0f) << 2) | ((src[i + 2] & 0xc0) >> 6)];
+
+            *iter++ = alpha[src[i + 2] & 0x3f];
+
+        }
+
+    /* Bourrage final ? */
+
+    if (i < input->len)
+    {
+        *iter++ = alpha[(src[i] >> 2) & 0x3f];
+
+        if (i == (input->len - 1))
+        {
+            *iter++ = alpha[((src[i] & 0x03) << 4)];
+            *iter++ = '=';
+        }
+        else
+        {
+            *iter++ = alpha[((src[i] & 0x03) << 4) | ((src[i + 1] & 0xf0) >> 4)];
+            *iter++ = alpha[((src[i + 1] & 0x0f) << 2)];
+        }
+
+        *iter++ = '=';
+    }
+
+    *iter = 0x00;
+
+ exit:
+
+    return result;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : input    = données d'entrée à traiter.                       *
+*                output   = données sortantes constituées. [OUT]              *
+*                                                                             *
+*  Description : Procède à l'encodage par défaut d'un contenu en base 64.     *
+*                                                                             *
+*  Retour      : Bilan de l'opération : true en cas de réussite, false sinon. *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+bool base64_encode(const sized_binary_t *input, sized_string_t *output)
+{
+    bool result;                            /* Bilan à retourner           */
+
+    const sized_string_t alphabet = {
+        .static_data = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
+        .len = 64
+    };
+
+    result = _base64_encode(input, output, &alphabet);
+
+    return result;
+
+}
diff --git a/plugins/encodings/base64.h b/plugins/encodings/base64.h
new file mode 100644
index 0000000..64ddccd
--- /dev/null
+++ b/plugins/encodings/base64.h
@@ -0,0 +1,43 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.h - prototypes pour les calculs d'encodages en base 64
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_BASE64_H
+#define _PLUGINS_ENCODINGS_BASE64_H
+
+
+#include <stdbool.h>
+
+
+#include <common/szstr.h>
+
+
+
+/* Procède à l'encodage d'un contenu en base 64. */
+bool _base64_encode(const sized_binary_t *, sized_string_t *, const sized_string_t *);
+
+/* Procède à l'encodage par défaut d'un contenu en base 64. */
+bool base64_encode(const sized_binary_t *, sized_string_t *);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_BASE64_H */
diff --git a/plugins/encodings/core.c b/plugins/encodings/core.c
new file mode 100644
index 0000000..2ece208
--- /dev/null
+++ b/plugins/encodings/core.c
@@ -0,0 +1,89 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * core.c - prototypes pour le calcul d'encodages
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+
+#include "core.h"
+
+
+#include <analysis/scan/core.h>
+#include <plugins/self.h>
+
+
+#ifdef INCLUDE_PYTHON3_BINDINGS
+#   include "python/module.h"
+#   include "python/rost/module.h"
+#endif
+#include "rost/base64.h"
+
+
+#ifdef INCLUDE_PYTHON3_BINDINGS_
+#   define PG_REQ RL("PyChrysalide")
+#else
+#   define PG_REQ NO_REQ
+#endif
+
+
+
+DEFINE_CHRYSALIDE_PLUGIN("Encodings", "Special encoding methods for binaries",
+                         PACKAGE_VERSION, CHRYSALIDE_WEBSITE("doc/formats"),
+                         PG_REQ, AL(PGA_PLUGIN_INIT));
+
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : plugin = greffon à manipuler.                                *
+*                                                                             *
+*  Description : Prend acte du chargement du greffon.                         *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+G_MODULE_EXPORT bool chrysalide_plugin_init(GPluginModule *plugin)
+{
+    bool result;                            /* Bilan à retourner           */
+    GScanTokenModifier *modifier;           /* Modificateur à enregistrer  */
+
+    modifier = g_scan_base64_modifier_new();
+
+    result = register_scan_token_modifier(modifier);
+
+    g_object_unref(G_OBJECT(modifier));
+
+    if (!result) goto exit;
+
+#ifdef INCLUDE_PYTHON3_BINDINGS
+
+    result = add_encodings_module_to_python_module();
+
+    if (result)
+        result = register_encodings_rost_modifiers();
+#endif
+
+ exit:
+
+    return result;
+
+}
diff --git a/plugins/encodings/core.h b/plugins/encodings/core.h
new file mode 100644
index 0000000..75a6e73
--- /dev/null
+++ b/plugins/encodings/core.h
@@ -0,0 +1,38 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * core.h - prototypes pour le calcul d'encodages
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_CORE_H
+#define _PLUGINS_ENCODINGS_CORE_H
+
+
+#include <plugins/plugin.h>
+#include <plugins/plugin-int.h>
+
+
+
+/* Prend acte du chargement du greffon. */
+G_MODULE_EXPORT bool chrysalide_plugin_init(GPluginModule *);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_CORE_H */
diff --git a/plugins/encodings/python/Makefile.am b/plugins/encodings/python/Makefile.am
new file mode 100644
index 0000000..f999f9f
--- /dev/null
+++ b/plugins/encodings/python/Makefile.am
@@ -0,0 +1,20 @@
+
+noinst_LTLIBRARIES = libencodingspython.la
+
+libencodingspython_la_SOURCES =				\
+	base64.h base64.c						\
+	module.h module.c
+
+libencodingspython_la_LIBADD =	 			\
+        rost/libencodingspythonrost.la
+
+libencodingspython_la_CFLAGS = $(TOOLKIT_CFLAGS) $(LIBXML_CFLAGS) $(LIBPYTHON_CFLAGS) $(LIBPYGOBJECT_CFLAGS) \
+	-I$(top_srcdir)/src -DNO_IMPORT_PYGOBJECT
+
+
+devdir = $(includedir)/chrysalide/$(subdir)
+
+dev_HEADERS = $(libencodingspython_la_SOURCES:%c=)
+
+
+SUBDIRS = rost
diff --git a/plugins/encodings/python/base64.c b/plugins/encodings/python/base64.c
new file mode 100644
index 0000000..3bc84a2
--- /dev/null
+++ b/plugins/encodings/python/base64.c
@@ -0,0 +1,139 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.c - équivalent Python du fichier "plugins/encodings/base64.c"
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#include "base64.h"
+
+
+#include <pygobject.h>
+
+
+#include <plugins/pychrysalide/access.h>
+#include <plugins/pychrysalide/helpers.h>
+
+
+#include "../base64.h"
+
+
+
+/* Procède à l'encodage d'un contenu en base 64. */
+static PyObject *py_encodings_base64_encode(PyObject *, PyObject *);
+
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : self = objet Python concerné par l'appel.                    *
+*                args = paramètre à récupérer pour le traitement.             *
+*                                                                             *
+*  Description : Procède à l'encodage d'un contenu en base 64.                *
+*                                                                             *
+*  Retour      : None ou données sortantes constituées.                       *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static PyObject *py_encodings_base64_encode(PyObject *self, PyObject *args)
+{
+    PyObject *result;                       /* Valeur à retourner          */
+    sized_string_t alphabet;                /* Alphabet de traitement ?    */
+    Py_ssize_t alen;                        /* Taille de cet alphabet      */
+    sized_binary_t input;                   /* Données à traiter           */
+    Py_ssize_t ilen;                        /* Quantité de ces données     */
+    int ret;                                /* Bilan de lecture des args.  */
+    sized_string_t output;                  /* Données encodées à exporter */
+    bool status;                            /* Bilan de l'opération        */
+
+#define ENCODINGS_BASE64_ENCODE_METHOD PYTHON_METHOD_DEF                \
+(                                                                       \
+    base64_encode, "input, /, alphabet=None",                           \
+    METH_VARARGS, py_encodings,                                         \
+    "Encode a given content using Base64.\n"                            \
+    "\n"                                                                \
+    "The *input* argument has to be provided as bytes. The optional"    \
+    " *alphabet* is expected to be a 64-byte array if defined.\n"       \
+    "\n"                                                                \
+    "The returned value is bytes or *None* in case of error."           \
+)
+
+    result = NULL;
+
+    alphabet.data = NULL;
+    alen = 0;
+
+    ret = PyArg_ParseTuple(args, "s#|s#", &input.static_data, &ilen, &alphabet.static_data, &alen);
+    if (!ret) goto exit;
+
+    alphabet.len = alen;
+    input.len = ilen;
+
+    if (alphabet.len > 0)
+        status = _base64_encode(&input, &output, &alphabet);
+    else
+        status = base64_encode(&input, &output);
+
+    if (status)
+    {
+        result = PyBytes_FromStringAndSize(output.data, output.len);
+        exit_szstr(&output);
+    }
+    else
+    {
+        result = Py_None;
+        Py_INCREF(result);
+    }
+
+ exit:
+
+    return result;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : super = module dont la définition est à compléter.           *
+*                                                                             *
+*  Description : Définit une extension du module 'encodings' à compléter.     *
+*                                                                             *
+*  Retour      : Bilan de l'opération.                                        *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+bool populate_encodings_module_with_base64(PyObject *super)
+{
+    bool result;                            /* Bilan à retourner           */
+
+    static PyMethodDef py_base64_methods[] = {
+        ENCODINGS_BASE64_ENCODE_METHOD,
+        { NULL }
+    };
+
+    result = register_python_module_methods(super, py_base64_methods);
+
+    return result;
+
+}
diff --git a/plugins/encodings/python/base64.h b/plugins/encodings/python/base64.h
new file mode 100644
index 0000000..1e08cf6
--- /dev/null
+++ b/plugins/encodings/python/base64.h
@@ -0,0 +1,39 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.h - équivalent Python du fichier "plugins/encodings/base64.h"
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_PYTHON_BASE64_H
+#define _PLUGINS_ENCODINGS_PYTHON_BASE64_H
+
+
+#include <Python.h>
+#include <stdbool.h>
+
+
+
+/* Définit une extension du module 'encodings' à compléter. */
+bool populate_encodings_module_with_base64(PyObject *);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_PYTHON_BASE64_H */
diff --git a/plugins/encodings/python/module.c b/plugins/encodings/python/module.c
new file mode 100644
index 0000000..454159b
--- /dev/null
+++ b/plugins/encodings/python/module.c
@@ -0,0 +1,87 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * module.c - intégration du répertoire encodings en tant que module
+ *
+ * Copyright (C) 2020 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#include "module.h"
+
+
+#include <assert.h>
+#include <Python.h>
+
+
+#include <plugins/pychrysalide/access.h>
+#include <plugins/pychrysalide/helpers.h>
+
+
+#include "base64.h"
+
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : -                                                            *
+*                                                                             *
+*  Description : Ajoute le module 'plugins.encodings' au module Python.       *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+bool add_encodings_module_to_python_module(void)
+{
+    bool result;                            /* Bilan à retourner           */
+    PyObject *super;                        /* Module à compléter          */
+    PyObject *module;                       /* Sous-module mis en place    */
+
+#define PYCHRYSALIDE_PLUGINS_ENCODINGS_DOC                  \
+    "encodings is a module providing a few implementations" \
+    " of algorithms used to encode data."
+
+    static PyModuleDef py_chrysalide_encodings_module = {
+
+        .m_base = PyModuleDef_HEAD_INIT,
+
+        .m_name = "pychrysalide.plugins.encodings",
+        .m_doc = PYCHRYSALIDE_PLUGINS_ENCODINGS_DOC,
+
+        .m_size = -1,
+
+    };
+
+    result = false;
+
+    super = get_access_to_python_module("pychrysalide.plugins");
+
+    module = build_python_module(super, &py_chrysalide_encodings_module);
+
+    result = (module != NULL);
+
+    if (result) result = populate_encodings_module_with_base64(module);
+
+    assert(result);
+
+    return result;
+
+}
diff --git a/plugins/encodings/python/module.h b/plugins/encodings/python/module.h
new file mode 100644
index 0000000..e7dd812
--- /dev/null
+++ b/plugins/encodings/python/module.h
@@ -0,0 +1,38 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * module.h - prototypes pour l'intégration du répertoire encodings en tant que module
+ *
+ * Copyright (C) 2020 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_PYTHON_MODULE_H
+#define _PLUGINS_ENCODINGS_PYTHON_MODULE_H
+
+
+#include <stdbool.h>
+
+
+
+/* Ajoute le module 'plugins.encodings' au module Python. */
+bool add_encodings_module_to_python_module(void);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_PYTHON_MODULE_H */
diff --git a/plugins/encodings/python/rost/Makefile.am b/plugins/encodings/python/rost/Makefile.am
new file mode 100644
index 0000000..a5a2969
--- /dev/null
+++ b/plugins/encodings/python/rost/Makefile.am
@@ -0,0 +1,14 @@
+
+noinst_LTLIBRARIES = libencodingspythonrost.la
+
+libencodingspythonrost_la_SOURCES = 		\
+	base64.h base64.c						\
+	module.h module.c
+
+libencodingspythonrost_la_CFLAGS = $(TOOLKIT_CFLAGS) $(LIBXML_CFLAGS) $(LIBPYTHON_CFLAGS) $(LIBPYGOBJECT_CFLAGS) \
+	-I$(top_srcdir)/src -DNO_IMPORT_PYGOBJECT
+
+
+devdir = $(includedir)/chrysalide-$(subdir)
+
+dev_HEADERS = $(libencodingspythonrost_la_SOURCES:%c=)
diff --git a/plugins/encodings/python/rost/base64.c b/plugins/encodings/python/rost/base64.c
new file mode 100644
index 0000000..57bba76
--- /dev/null
+++ b/plugins/encodings/python/rost/base64.c
@@ -0,0 +1,211 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.c - équivalent Python du fichier "plugins/encodings/rost/base64.c"
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#include "base64.h"
+
+
+#include <pygobject.h>
+
+
+#include <i18n.h>
+#include <plugins/pychrysalide/access.h>
+#include <plugins/pychrysalide/helpers.h>
+#include <plugins/pychrysalide/analysis/scan/patterns/modifier.h>
+
+
+#include "../../rost/base64.h"
+
+
+
+CREATE_DYN_CONSTRUCTOR(scan_base64_modifier, G_TYPE_SCAN_BASE64_MODIFIER);
+
+/* Initialise une instance sur la base du dérivé de GObject. */
+static int py_scan_base64_modifier_init(PyObject *, PyObject *, PyObject *);
+
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : self = objet à initialiser (théoriquement).                  *
+*                args = arguments fournis à l'appel.                          *
+*                kwds = arguments de type key=val fournis.                    *
+*                                                                             *
+*  Description : Initialise une instance sur la base du dérivé de GObject.    *
+*                                                                             *
+*  Retour      : 0.                                                           *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static int py_scan_base64_modifier_init(PyObject *self, PyObject *args, PyObject *kwds)
+{
+    int ret;                                /* Bilan de lecture des args.  */
+
+#define SCAN_BASE64_MODIFIER_DOC                                    \
+    "The *Base64Modifier* class transforms a byte pattern into its" \
+    " base64 encoded form.\n"                                       \
+    "\n"                                                            \
+    "Instances can be created using the following constructor:\n"   \
+    "\n"                                                            \
+    "    Base64Modifier()"
+
+    /* Initialisation d'un objet GLib */
+
+    ret = forward_pygobjet_init(self);
+    if (ret == -1) return -1;
+
+    return 0;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : -                                                            *
+*                                                                             *
+*  Description : Fournit un accès à une définition de type à diffuser.        *
+*                                                                             *
+*  Retour      : Définition d'objet pour Python.                              *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+PyTypeObject *get_python_scan_base64_modifier_type(void)
+{
+    static PyMethodDef py_scan_base64_modifier_methods[] = {
+        { NULL }
+    };
+
+    static PyGetSetDef py_scan_base64_modifier_getseters[] = {
+        { NULL }
+    };
+
+    static PyTypeObject py_scan_base64_modifier_type = {
+
+        PyVarObject_HEAD_INIT(NULL, 0)
+
+        .tp_name        = "pychrysalide.analysis.scan.patterns.modifiers.Base64Modifier",
+        .tp_basicsize   = sizeof(PyGObject),
+
+        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+
+        .tp_doc         = SCAN_BASE64_MODIFIER_DOC,
+
+        .tp_methods     = py_scan_base64_modifier_methods,
+        .tp_getset      = py_scan_base64_modifier_getseters,
+
+        .tp_init        = py_scan_base64_modifier_init,
+        .tp_new         = py_scan_base64_modifier_new,
+
+    };
+
+    return &py_scan_base64_modifier_type;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : -                                                            *
+*                                                                             *
+*  Description : Prend en charge l'objet 'pychrysalide....Base64Modifier'.    *
+*                                                                             *
+*  Retour      : Bilan de l'opération.                                        *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+bool ensure_python_scan_base64_modifier_is_registered(void)
+{
+    PyTypeObject *type;                     /* Type Python Base64Modifier */
+    PyObject *module;                       /* Module à recompléter        */
+    PyObject *dict;                         /* Dictionnaire du module      */
+
+    type = get_python_scan_base64_modifier_type();
+
+    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
+    {
+        module = get_access_to_python_module("pychrysalide.analysis.scan.patterns.modifiers");
+
+        dict = PyModule_GetDict(module);
+
+        if (!ensure_python_scan_token_modifier_is_registered())
+            return false;
+
+        if (!register_class_for_pygobject(dict, G_TYPE_SCAN_BASE64_MODIFIER, type))
+            return false;
+
+    }
+
+    return true;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : arg = argument quelconque à tenter de convertir.             *
+*                dst = destination des valeurs récupérées en cas de succès.   *
+*                                                                             *
+*  Description : Tente de convertir en transformation par encodage en Base64. *
+*                                                                             *
+*  Retour      : Bilan de l'opération, voire indications supplémentaires.     *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+int convert_to_scan_base64_modifier(PyObject *arg, void *dst)
+{
+    int result;                             /* Bilan à retourner           */
+
+    result = PyObject_IsInstance(arg, (PyObject *)get_python_scan_base64_modifier_type());
+
+    switch (result)
+    {
+        case -1:
+            /* L'exception est déjà fixée par Python */
+            result = 0;
+            break;
+
+        case 0:
+            PyErr_SetString(PyExc_TypeError, "unable to convert the provided argument to Base64 modifier");
+            break;
+
+        case 1:
+            *((GScanBase64Modifier **)dst) = G_SCAN_BASE64_MODIFIER(pygobject_get(arg));
+            break;
+
+        default:
+            assert(false);
+            break;
+
+    }
+
+    return result;
+
+}
diff --git a/plugins/encodings/python/rost/base64.h b/plugins/encodings/python/rost/base64.h
new file mode 100644
index 0000000..c4459e9
--- /dev/null
+++ b/plugins/encodings/python/rost/base64.h
@@ -0,0 +1,45 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.h - équivalent Python du fichier "plugins/encodings/rost/base64.h"
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_PYTHON_ROST_BASE64_H
+#define _PLUGINS_ENCODINGS_PYTHON_ROST_BASE64_H
+
+
+#include <Python.h>
+#include <stdbool.h>
+
+
+
+/* Fournit un accès à une définition de type à diffuser. */
+PyTypeObject *get_python_scan_base64_modifier_type(void);
+
+/* Prend en charge l'objet 'pychrysalide....Base64Modifier'. */
+bool ensure_python_scan_base64_modifier_is_registered(void);
+
+/* Tente de convertir en transformation par encodage en Base64. */
+int convert_to_scan_base64_modifier(PyObject *, void *);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_PYTHON_ROST_BASE64_H */
diff --git a/plugins/encodings/python/rost/module.c b/plugins/encodings/python/rost/module.c
new file mode 100644
index 0000000..6bd7187
--- /dev/null
+++ b/plugins/encodings/python/rost/module.c
@@ -0,0 +1,63 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * module.c - intégration du répertoire rost en tant que module
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#include "module.h"
+
+
+#include <Python.h>
+
+
+#include <plugins/pychrysalide/access.h>
+#include <plugins/pychrysalide/helpers.h>
+
+
+#include "base64.h"
+
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : -                                                            *
+*                                                                             *
+*  Description : Intègre les modificateurs pour ROST.                         *
+*                                                                             *
+*  Retour      : Bilan de l'opération.                                        *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+bool register_encodings_rost_modifiers(void)
+{
+    bool result;                            /* Bilan à retourner           */
+
+    result = true;
+
+    if (result) result = ensure_python_scan_base64_modifier_is_registered();
+
+    assert(result);
+
+    return result;
+
+}
diff --git a/plugins/encodings/python/rost/module.h b/plugins/encodings/python/rost/module.h
new file mode 100644
index 0000000..ba021cb
--- /dev/null
+++ b/plugins/encodings/python/rost/module.h
@@ -0,0 +1,38 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * module.h - prototypes pour l'intégration du répertoire rost en tant que module
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_PYTHON_ROST_MODULE_H
+#define _PLUGINS_ENCODINGS_PYTHON_ROST_MODULE_H
+
+
+#include <stdbool.h>
+
+
+
+/* Intègre les modificateurs pour ROST. */
+bool register_encodings_rost_modifiers(void);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_PYTHON_ROST_MODULE_H */
diff --git a/plugins/encodings/rost/Makefile.am b/plugins/encodings/rost/Makefile.am
new file mode 100644
index 0000000..efca690
--- /dev/null
+++ b/plugins/encodings/rost/Makefile.am
@@ -0,0 +1,12 @@
+
+noinst_LTLIBRARIES = libencodingsrost.la
+
+libencodingsrost_la_SOURCES =				\
+	base64.h base64.c
+
+libencodingsrost_la_CFLAGS = $(TOOLKIT_CFLAGS) $(LIBXML_CFLAGS) -I$(top_srcdir)/src
+
+
+devdir = $(includedir)/chrysalide-$(subdir)
+
+dev_HEADERS = $(libencodingsrost_la_SOURCES:%c=)
diff --git a/plugins/encodings/rost/base64.c b/plugins/encodings/rost/base64.c
new file mode 100644
index 0000000..0c5bafb
--- /dev/null
+++ b/plugins/encodings/rost/base64.c
@@ -0,0 +1,513 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.c - transormation en encodage Base64
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+
+#include "base64.h"
+
+
+#include <assert.h>
+#include <malloc.h>
+#include <stdint.h>
+#include <string.h>
+
+
+#include <analysis/scan/patterns/modifier-int.h>
+
+
+#include "../base64.h"
+
+
+
+/* ----------------------- RECHERCHE D'UN MOTIF DE TEXTE BRUT ----------------------- */
+
+
+/* Initialise la classe des transmissions en encodage Base64. */
+static void g_scan_base64_modifier_class_init(GScanBase64ModifierClass *klass);
+
+/* Initialise une instance de transmission en encodage Base64. */
+static void g_scan_base64_modifier_init(GScanBase64Modifier *);
+
+/* Supprime toutes les références externes. */
+static void g_scan_base64_modifier_dispose(GScanBase64Modifier *);
+
+/* Procède à la libération totale de la mémoire. */
+static void g_scan_base64_modifier_finalize(GScanBase64Modifier *);
+
+
+
+/* --------------------- IMPLEMENTATION DES FONCTIONS DE CLASSE --------------------- */
+
+
+/* Fournit le nom d'appel d'un modificateur pour motif. */
+static char *g_scan_base64_modifier_get_name(const GScanBase64Modifier *);
+
+/* Finalise l'encoddage en Base64 d'un motif transformé. */
+static void strip_base64_modifier_output(const sized_binary_t *, size_t, sized_binary_t *);
+
+/* Transforme une séquence d'octets pour motif de recherche. */
+static bool g_scan_base64_modifier_transform(const GScanBase64Modifier *, const sized_binary_t *, size_t, sized_binary_t **, size_t *);
+
+/* Détermine si un argument est bien toléré par un modificateur. */
+static bool g_scan_base64_modifier_can_handle_arg(const GScanBase64Modifier *, const modifier_arg_t *);
+
+/* Transforme une séquence d'octets pour motif de recherche. */
+static bool g_scan_base64_modifier_transform_with_arg(const GScanBase64Modifier *, const sized_binary_t *, size_t, const modifier_arg_t *, sized_binary_t **, size_t *);
+
+
+
+/* ---------------------------------------------------------------------------------- */
+/*                         RECHERCHE D'UN MOTIF DE TEXTE BRUT                         */
+/* ---------------------------------------------------------------------------------- */
+
+
+/* Indique le type défini pour une transormation en encodage Base64. */
+G_DEFINE_TYPE(GScanBase64Modifier, g_scan_base64_modifier, G_TYPE_SCAN_TOKEN_MODIFIER);
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : klass = classe à initialiser.                                *
+*                                                                             *
+*  Description : Initialise la classe des transmissions en encodage Base64.   *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static void g_scan_base64_modifier_class_init(GScanBase64ModifierClass *klass)
+{
+    GObjectClass *object;                   /* Autre version de la classe  */
+    GScanTokenModifierClass *modifier;      /* Version de classe parente   */
+
+    object = G_OBJECT_CLASS(klass);
+
+    object->dispose = (GObjectFinalizeFunc/* ! */)g_scan_base64_modifier_dispose;
+    object->finalize = (GObjectFinalizeFunc)g_scan_base64_modifier_finalize;
+
+    modifier = G_SCAN_TOKEN_MODIFIER_CLASS(klass);
+
+    modifier->get_name = (get_scan_modifier_name_fc)g_scan_base64_modifier_get_name;
+
+    modifier->transform = (transform_scan_token_fc)g_scan_base64_modifier_transform;
+    modifier->can_handle = (can_token_modifier_handle_arg)g_scan_base64_modifier_can_handle_arg;
+    modifier->transform_with = (transform_scan_token_with_fc)g_scan_base64_modifier_transform_with_arg;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = instance à initialiser.                           *
+*                                                                             *
+*  Description : Initialise une instance de transmission en encodage Base64.  *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static void g_scan_base64_modifier_init(GScanBase64Modifier *modifier)
+{
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = instance d'objet GLib à traiter.                  *
+*                                                                             *
+*  Description : Supprime toutes les références externes.                     *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static void g_scan_base64_modifier_dispose(GScanBase64Modifier *modifier)
+{
+    G_OBJECT_CLASS(g_scan_base64_modifier_parent_class)->dispose(G_OBJECT(modifier));
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = instance d'objet GLib à traiter.                  *
+*                                                                             *
+*  Description : Procède à la libération totale de la mémoire.                *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static void g_scan_base64_modifier_finalize(GScanBase64Modifier *modifier)
+{
+    G_OBJECT_CLASS(g_scan_base64_modifier_parent_class)->finalize(G_OBJECT(modifier));
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : -                                                            *
+*                                                                             *
+*  Description : Construit un modificateur livrant des encodages en Base64.   *
+*                                                                             *
+*  Retour      : Mécanisme mis en place.                                      *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+GScanTokenModifier *g_scan_base64_modifier_new(void)
+{
+    GScanTokenModifier *result;                    /* Structure à retourner       */
+
+    result = g_object_new(G_TYPE_SCAN_BASE64_MODIFIER, NULL);
+
+    return result;
+
+}
+
+
+
+/* ---------------------------------------------------------------------------------- */
+/*                       IMPLEMENTATION DES FONCTIONS DE CLASSE                       */
+/* ---------------------------------------------------------------------------------- */
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = modificateur à consulter.                         *
+*                                                                             *
+*  Description : Fournit le nom d'appel d'un modificateur pour motif.         *
+*                                                                             *
+*  Retour      : Désignation humaine.                                         *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static char *g_scan_base64_modifier_get_name(const GScanBase64Modifier *modifier)
+{
+    char *result;                           /* Désignation à retourner     */
+
+    result = strdup("base64");
+
+    return result;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : input  = encodage en Base64 obtenu.                          *
+*                skip   = nombre de caractères initiaux à sauter.             *
+*                output = encodage en Base64 final à conserver.               *
+*                                                                             *
+*  Description : Finalise l'encoddage en Base64 d'un motif transformé.        *
+*                                                                             *
+*  Retour      : -                                                            *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static void strip_base64_modifier_output(const sized_binary_t *input, size_t skip, sized_binary_t *output)
+{
+    size_t final_len;                       /* Taille de représentation    */
+
+    final_len = input->len;
+
+    if (final_len > 0 && input->bin_data[final_len - 1] == '=')
+    {
+        while (final_len > 0 && input->bin_data[final_len - 1] == '=')
+            final_len--;
+
+        final_len--;
+
+    }
+
+    if (skip > 0)
+        skip++;
+
+    assert(final_len >= skip);
+
+    final_len -= skip;
+
+    output->len = final_len;
+    output->bin_data = malloc(final_len * sizeof(bin_t));
+
+    memcpy(output->bin_data, input->static_bin_data + skip, final_len);
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = modificateur à solliciter.                        *
+*                src      = séquence d'octets à traiter.                      *
+*                scount   = quantité de ces séquences.                        *
+*                dest     = nouvelle(s) séquence(s) d'octets obtenue(s) [OUT] *
+*                dcount   = quantité de ces séquences.                        *
+*                                                                             *
+*  Description : Transforme une séquence d'octets pour motif de recherche.    *
+*                                                                             *
+*  Retour      : Bilan de l'opération : succès ou échec.                      *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static bool g_scan_base64_modifier_transform(const GScanBase64Modifier *modifier, const sized_binary_t *src, size_t scount, sized_binary_t **dest, size_t *dcount)
+{
+    bool result;                            /* Bilan d'opération à renvoyer*/
+    sized_binary_t *binary;                 /* Raccourci vers le stockage  */
+    size_t i;                               /* Boucle de parcours #1       */
+    const sized_binary_t *_src;             /* Source courante             */
+    sized_binary_t tmp_in;                  /* Manipulation des variations */
+    sized_binary_t tmp_out;                 /* Manipulation des variations */
+
+    result = true;
+
+    *dcount = scount * 3;
+    *dest = calloc(*dcount, sizeof(sized_binary_t));
+
+    binary = &(*dest)[0];
+
+    for (i = 0; i < scount && result; i++)
+    {
+        _src = src + i;
+
+        tmp_in.data = malloc((_src->len + 2) * sizeof(bin_t));
+
+        /**
+         * Explications sur la définition des modifications sur les
+         * encodages produits : https://www.leeholmes.com/searching-for-content-in-base-64-strings/
+         */
+
+        /* Encodage premier */
+
+        result = base64_encode(_src, &tmp_out);
+        if (!result) goto exit;
+
+        strip_base64_modifier_output(&tmp_out, 0, binary++);
+
+        exit_szstr(&tmp_out);
+
+        /* Variante 1 */
+
+        tmp_in.data[0] = ' ';
+        memcpy(&tmp_in.data[1], _src->static_bin_data, _src->len);
+
+        tmp_in.len = _src->len + 1;
+
+        result = base64_encode(&tmp_in, &tmp_out);
+        if (!result) goto exit;
+
+        strip_base64_modifier_output(&tmp_out, 1, binary++);
+
+        exit_szstr(&tmp_out);
+
+        /* Variante 2 */
+
+        tmp_in.data[0] = ' ';
+        tmp_in.data[2] = ' ';
+        memcpy(&tmp_in.data[2], _src->static_bin_data, _src->len);
+
+        tmp_in.len = _src->len + 2;
+
+        result = base64_encode(&tmp_in, &tmp_out);
+        if (!result) goto exit;
+
+        strip_base64_modifier_output(&tmp_out, 2, binary++);
+
+        exit_szstr(&tmp_out);
+
+ exit:
+
+        exit_szstr(&tmp_in);
+
+    }
+
+    if (!result)
+    {
+        for (i = 0; i < *dcount; i++)
+            exit_szstr(&(*dest)[i]);
+
+        free(*dest);
+
+        *dcount = 0;
+        *dest = NULL;
+
+    }
+
+    return result;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = modificateur à solliciter.                        *
+*                arg      = argument de personnalisation.                     *
+*                                                                             *
+*  Description : Détermine si un argument est bien toléré par un modificateur.*
+*                                                                             *
+*  Retour      : Bilan de la consultation : support ou non.                   *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static bool g_scan_base64_modifier_can_handle_arg(const GScanBase64Modifier *modifier, const modifier_arg_t *arg)
+{
+    bool result;                            /* Bilan d'opération à renvoyer*/
+
+    switch (arg->type)
+    {
+        case MAT_STRING:
+            result = (arg->value.string.len == 64);
+            break;
+
+        default:
+            result = false;
+
+    }
+
+    return result;
+
+}
+
+
+/******************************************************************************
+*                                                                             *
+*  Paramètres  : modifier = modificateur à solliciter.                        *
+*                src      = séquence d'octets à traiter.                      *
+*                scount   = quantité de ces séquences.                        *
+*                arg      = argument de personnalisation.                     *
+*                dest     = nouvelle(s) séquence(s) d'octets obtenue(s) [OUT] *
+*                dcount   = quantité de ces séquences.                        *
+*                                                                             *
+*  Description : Transforme une séquence d'octets pour motif de recherche.    *
+*                                                                             *
+*  Retour      : Bilan de l'opération : succès ou échec.                      *
+*                                                                             *
+*  Remarques   : -                                                            *
+*                                                                             *
+******************************************************************************/
+
+static bool g_scan_base64_modifier_transform_with_arg(const GScanBase64Modifier *modifier, const sized_binary_t *src, size_t scount, const modifier_arg_t *arg, sized_binary_t **dest, size_t *dcount)
+{
+    bool result;                            /* Bilan d'opération à renvoyer*/
+    sized_binary_t *binary;                 /* Raccourci vers le stockage  */
+    size_t i;                               /* Boucle de parcours #1       */
+    const sized_binary_t *_src;             /* Source courante             */
+    sized_binary_t tmp_in;                  /* Manipulation des variations */
+    sized_binary_t tmp_out;                 /* Manipulation des variations */
+
+    result = (arg->type == MAT_STRING);
+    if (!result) goto bad_arg;
+
+    *dcount = scount * 3;
+    *dest = calloc(*dcount, sizeof(sized_binary_t));
+
+    binary = &(*dest)[0];
+
+    for (i = 0; i < scount && result; i++)
+    {
+        _src = src + i;
+
+        tmp_in.data = malloc((_src->len + 2) * sizeof(bin_t));
+
+        /**
+         * Explications sur la définition des modifications sur les
+         * encodages produits : https://www.leeholmes.com/searching-for-content-in-base-64-strings/
+         */
+
+        /* Encodage premier */
+
+        result = _base64_encode(_src, &tmp_out, &arg->value.string);
+        if (!result) goto exit;
+
+        strip_base64_modifier_output(&tmp_out, 0, binary++);
+
+        exit_szstr(&tmp_out);
+
+        /* Variante 1 */
+
+        tmp_in.data[0] = ' ';
+        memcpy(&tmp_in.data[1], _src->static_bin_data, _src->len);
+
+        tmp_in.len = _src->len + 1;
+
+        result = _base64_encode(&tmp_in, &tmp_out, &arg->value.string);
+        if (!result) goto exit;
+
+        strip_base64_modifier_output(&tmp_out, 1, binary++);
+
+        exit_szstr(&tmp_out);
+
+        /* Variante 2 */
+
+        tmp_in.data[0] = ' ';
+        tmp_in.data[2] = ' ';
+        memcpy(&tmp_in.data[2], _src->static_bin_data, _src->len);
+
+        tmp_in.len = _src->len + 2;
+
+        result = _base64_encode(&tmp_in, &tmp_out, &arg->value.string);
+        if (!result) goto exit;
+
+        strip_base64_modifier_output(&tmp_out, 2, binary++);
+
+        exit_szstr(&tmp_out);
+
+ exit:
+
+        exit_szstr(&tmp_in);
+
+    }
+
+    if (!result)
+    {
+        for (i = 0; i < *dcount; i++)
+            exit_szstr(&(*dest)[i]);
+
+        free(*dest);
+
+        *dcount = 0;
+        *dest = NULL;
+
+    }
+
+ bad_arg:
+
+    return result;
+
+}
diff --git a/plugins/encodings/rost/base64.h b/plugins/encodings/rost/base64.h
new file mode 100644
index 0000000..646b945
--- /dev/null
+++ b/plugins/encodings/rost/base64.h
@@ -0,0 +1,58 @@
+
+/* Chrysalide - Outil d'analyse de fichiers binaires
+ * base64.h - prototypes pour la transormation en encodage Base64
+ *
+ * Copyright (C) 2023 Cyrille Bagard
+ *
+ *  This file is part of Chrysalide.
+ *
+ *  Chrysalide is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Chrysalide is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+
+#ifndef _PLUGINS_ENCODINGS_ROST_BASE64_H
+#define _PLUGINS_ENCODINGS_ROST_BASE64_H
+
+
+#include <glib-object.h>
+
+
+#include <analysis/scan/patterns/modifier.h>
+
+
+
+#define G_TYPE_SCAN_BASE64_MODIFIER            g_scan_base64_modifier_get_type()
+#define G_SCAN_BASE64_MODIFIER(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_SCAN_BASE64_MODIFIER, GScanBase64Modifier))
+#define G_IS_SCAN_BASE64_MODIFIER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_SCAN_BASE64_MODIFIER))
+#define G_SCAN_BASE64_MODIFIER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_SCAN_BASE64_MODIFIER, GScanBase64ModifierClass))
+#define G_IS_SCAN_BASE64_MODIFIER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_SCAN_BASE64_MODIFIER))
+#define G_SCAN_BASE64_MODIFIER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_SCAN_BASE64_MODIFIER, GScanBase64ModifierClass))
+
+
+/* Transormation en encodage Base64 (instance) */
+typedef GScanTokenModifier GScanBase64Modifier;
+
+/* Transormation en encodage Base64 (classe) */
+typedef GScanTokenModifierClass GScanBase64ModifierClass;
+
+
+/* Indique le type défini pour une transormation en encodage Base64. */
+GType g_scan_base64_modifier_get_type(void);
+
+/* Construit un modificateur livrant des encodages en Base64. */
+GScanTokenModifier *g_scan_base64_modifier_new(void);
+
+
+
+#endif  /* _PLUGINS_ENCODINGS_ROST_BASE64_H */
diff --git a/src/analysis/scan/patterns/modifiers/hex.c b/src/analysis/scan/patterns/modifiers/hex.c
index da992df..a50c67b 100644
--- a/src/analysis/scan/patterns/modifiers/hex.c
+++ b/src/analysis/scan/patterns/modifiers/hex.c
@@ -229,7 +229,7 @@ static bool g_scan_hex_modifier_transform(const GScanHexModifier *modifier, cons
     bool result;                            /* Bilan d'opération à renvoyer*/
     sized_binary_t *binary;                 /* Raccourci vers le stockage  */
     size_t i;                               /* Boucle de parcours #1       */
-    const sized_binary_t *_src;             /* SOurce courante             */
+    const sized_binary_t *_src;             /* Source courante             */
     size_t k;                               /* Boucle de parcours #2       */
 
     static char *alphabet = "0123456789abcdef";
diff --git a/src/analysis/scan/patterns/modifiers/plain.c b/src/analysis/scan/patterns/modifiers/plain.c
index 485bb23..6d3ed5d 100644
--- a/src/analysis/scan/patterns/modifiers/plain.c
+++ b/src/analysis/scan/patterns/modifiers/plain.c
@@ -229,7 +229,7 @@ static bool g_scan_plain_modifier_transform(const GScanPlainModifier *modifier,
     bool result;                            /* Bilan d'opération à renvoyer*/
     sized_binary_t *binary;                 /* Raccourci vers le stockage  */
     size_t i;                               /* Boucle de parcours          */
-    const sized_binary_t *_src;             /* SOurce courante             */
+    const sized_binary_t *_src;             /* Source courante             */
 
     result = true;
 
diff --git a/src/analysis/scan/patterns/modifiers/rev.c b/src/analysis/scan/patterns/modifiers/rev.c
index 8b931bd..afdf567 100644
--- a/src/analysis/scan/patterns/modifiers/rev.c
+++ b/src/analysis/scan/patterns/modifiers/rev.c
@@ -229,7 +229,7 @@ static bool g_scan_reverse_modifier_transform(const GScanReverseModifier *modifi
     bool result;                            /* Bilan d'opération à renvoyer*/
     sized_binary_t *binary;                 /* Raccourci vers le stockage  */
     size_t i;                               /* Boucle de parcours #1       */
-    const sized_binary_t *_src;             /* SOurce courante             */
+    const sized_binary_t *_src;             /* Source courante             */
     size_t k;                               /* Boucle de parcours #2       */
 
     result = true;
diff --git a/src/analysis/scan/patterns/modifiers/xor.c b/src/analysis/scan/patterns/modifiers/xor.c
index 65194ab..4ec20a4 100644
--- a/src/analysis/scan/patterns/modifiers/xor.c
+++ b/src/analysis/scan/patterns/modifiers/xor.c
@@ -239,7 +239,7 @@ static bool g_scan_xor_modifier_transform(const GScanXorModifier *modifier, cons
     bool result;                            /* Bilan d'opération à renvoyer*/
     sized_binary_t *binary;                 /* Raccourci vers le stockage  */
     size_t i;                               /* Boucle de parcours #1       */
-    const sized_binary_t *_src;             /* SOurce courante             */
+    const sized_binary_t *_src;             /* Source courante             */
     long long x;                            /* Boucle de parcours #2       */
     size_t k;                               /* Boucle de parcours #3       */
 
@@ -330,7 +330,7 @@ static bool g_scan_xor_modifier_transform_with_arg(const GScanXorModifier *modif
     bool result;                            /* Bilan d'opération à renvoyer*/
     sized_binary_t *binary;                 /* Raccourci vers le stockage  */
     size_t i;                               /* Boucle de parcours #1       */
-    const sized_binary_t *_src;             /* SOurce courante             */
+    const sized_binary_t *_src;             /* Source courante             */
     long long x;                            /* Boucle de parcours #2       */
     size_t k;                               /* Boucle de parcours #3       */
 
diff --git a/src/common/szstr.h b/src/common/szstr.h
index aad1920..406a9f1 100644
--- a/src/common/szstr.h
+++ b/src/common/szstr.h
@@ -38,8 +38,13 @@
 typedef struct _sized_string_t
 {
     union {
+
         const char *static_data;        /* Données non modifiées       */
         char *data;                     /* Chaîne de caractères        */
+
+        const bin_t *static_bin_data;   /* Données brutes non modifiées*/
+        bin_t *bin_data;                /* Données brutes              */
+
     };
 
     size_t len;                         /* Taille correspondante       */
diff --git a/tests/analysis/scan/pyapi.py b/tests/analysis/scan/pyapi.py
index 8764074..2aaaeec 100644
--- a/tests/analysis/scan/pyapi.py
+++ b/tests/analysis/scan/pyapi.py
@@ -73,6 +73,7 @@ class TestRostPythonAPI(ChrysalideTestCase):
 
         self.assertEqual(source, transformed[0])
 
+
         mod = find_token_modifiers_for_name('hex')
         self.assertIsNotNone(mod)
 
@@ -81,6 +82,7 @@ class TestRostPythonAPI(ChrysalideTestCase):
 
         self.assertEqual(binascii.hexlify(source), transformed[0])
 
+
         mod = find_token_modifiers_for_name('rev')
         self.assertIsNotNone(mod)
 
@@ -90,6 +92,18 @@ class TestRostPythonAPI(ChrysalideTestCase):
         self.assertEqual(source[::-1], transformed[0])
 
 
+        mod = find_token_modifiers_for_name('base64')
+        self.assertIsNotNone(mod)
+
+        source = b'ABC'
+        transformed = mod.transform(source)
+
+        self.assertEqual(len(transformed), 3)
+        self.assertEqual(transformed[0], b'QUJD')
+        self.assertEqual(transformed[1], b'FCQw')
+        self.assertEqual(transformed[2], b'BQkM')
+
+
     def testBytePatternModifiersAPI(self):
         """Validate the API for pattern modifiers."""
 
diff --git a/plugins/pychrysalide/analysis/scan/patterns/modifiers/xor.c b/plugins/pychrysalide/analysis/scan/patterns/modifiers/xor.c
index 1280f2f..17a32f4 100644
--- a/plugins/pychrysalide/analysis/scan/patterns/modifiers/xor.c
+++ b/plugins/pychrysalide/analysis/scan/patterns/modifiers/xor.c
@@ -1,6 +1,6 @@
 
 /* Chrysalide - Outil d'analyse de fichiers binaires
  * hex.c - équivalent Python du fichier "analysis/scan/patterns/modifiers/hex.c"
  *
  * Copyright (C) 2023 Cyrille Bagard
  *
@@ -63,7 +63,7 @@ static int py_scan_xor_modifier_init(PyObject *self, PyObject *args, PyObject *k
 {
     int ret;                                /* Bilan de lecture des args.  */
 
-#define SCAN_HEX_MODIFIER_DOC                                               \
+#define SCAN_XOR_MODIFIER_DOC                                               \
     "The *XorModifier* class transforms a byte pattern by XORing bytes.\n"  \
     "\n"                                                                    \
     "Instances can be created using the following constructor:\n"           \
@@ -111,7 +111,7 @@ PyTypeObject *get_python_scan_xor_modifier_type(void)
 
         .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
 
-        .tp_doc         = SCAN_HEX_MODIFIER_DOC,
+        .tp_doc         = SCAN_XOR_MODIFIER_DOC,
 
         .tp_methods     = py_scan_xor_modifier_methods,
         .tp_getset      = py_scan_xor_modifier_getseters,
@@ -1,6 +1,6 @@
 
 /* Chrysalide - Outil d'analyse de fichiers binaires
- * hex.c - équivalent Python du fichier "analysis/scan/patterns/modifiers/hex.c"
+ * xor.c - équivalent Python du fichier "analysis/scan/patterns/modifiers/xor.c"
  *
  * Copyright (C) 2023 Cyrille Bagard
-- 
cgit v0.11.2-87-g4458